#include <stdio.h>   // standard I/O
#include <stdbool.h> // booleans
#include <stdint.h>  // Standard Integer Types

// Purpose: exhaustive test of binary search bug demonstrating overflow

int calcMid(int high, int low) {	// Function to calculate mid-point in a binary search
	int mid;

	mid = (int8_t)(low + high) / 2;  // FIX THIS BUG !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	//            ----------------

	/*
	CHANGE the above formula (low + high) / 2 to prevent the possibility of (low + high) intermediate value overflow
	mid = (int8_t) is necessary to force overflow only for illustration purposes here,
		  overflow will occur in this demo when (low + high) is > 127 because 127 is the max value of an 8-bit integer, data type (int8_t). The test is still valid with (int8_t) casting the intermediate value to an 8-bit integer before it is divided by 2. Normally, overflow would not occur until the sum exceeded 32,767 (16-bit int) or 2.147Gb (32-bit) but that would make for a very long demo.
	
	Microsoft's C compiler has built-in overflow protection for intermediate values: the result of 8-bit terms in an intermediate formula are stored as 16-bit, 16-bit as 32-bit, 32-bit as 64-bit. It is safe but inefficient. But all C compilers do not do this. Overflow protection should never be assumed.
	*/

	return mid;
}
/*

(low+high) will overflow MAX value if both are in upper half of range. e.g.
0------------------low--------high-----MAX    <== simulated integer range of values, calc new mid point

0------------------low
 PLUS
0-----------------------------high
 EQUALS
0------------------low-----------------------------high
0 -------------------------------------MAX============= oops! overflow

Hint: instead of finding the average of low+high as in the bug version, find the middle point between low and high.
0------------------low--------high-----MAX
				   |-------------|
1st find mid point        ^        between low and high
*/

int errorsFound = 0;
int totalSearches = 0;
int countSuccessfulSearches;
int countUnsuccessfulSearches;
const int RANGE_LOW = 1;		 // minimum search range
const int RANGE_HIGH = 71;	 // maximum search range 
const int OVER_THE_EDGE = 3; // value to test below rangeLow and above rangeHigh, i.e. beyond the edge conditions
// Search range should NEVER be an exponent of 2, e.g. 0-127 or 128 values. Errors can hide in a binary system when things are a factor of 2.


bool binarySearchTest(int target);  // Binary search function prototype

int main()
{
	printf("Searching for values within and just outside the range %d <-> %d\n\n", RANGE_LOW, RANGE_HIGH);

	// search for each possible value in the range and a few values just outside range to check edge conditions

	for (int i = RANGE_LOW - OVER_THE_EDGE; i <= RANGE_HIGH + OVER_THE_EDGE; i++)
	{
		binarySearchTest(i);
		printf("\n");
	}
	printf("\nTest range  : %5d <-> %5d\n", RANGE_LOW, RANGE_HIGH);
	printf("\nSearch range: %5d <-> %5d\n", RANGE_LOW - OVER_THE_EDGE, RANGE_HIGH + OVER_THE_EDGE);
	printf("\nSearch errors: %d \tUnsuccessful searches: %d (%d are expected)\n", errorsFound, countUnsuccessfulSearches, OVER_THE_EDGE * 2);
	printf("\n%.2f is average number of tries in %d successful searches over %d items\n", (float)totalSearches / countSuccessfulSearches, countSuccessfulSearches, RANGE_HIGH - RANGE_LOW + 1);

	printf("\npress Enter to end.");
	getchar(); // pause terminal window

	return errorsFound; // should be zero for normal end of job
}

/*  binary search for a target value within a range of values */
bool binarySearchTest(int target)
{
	int
		low = RANGE_LOW,	// initial lowest search range
		high = RANGE_HIGH,	// initial highest search range 
		mid = low;			// mis is middle point of search range, prime with lowest range.
	int
		prevLow = low,		// previous values for quality evaluation
		prevHigh = high,
		prevMid = high;

	int	counter = 0;		// iterations of search

	// *****************************************************************************************************
	printf("low  mid  high  --> searching for mid = %d", target);
	// *****************************************************************************************************

	do  // search between low & high for target
	{
		counter++;			// iterations for current search

		prevLow = low; 		// previous values for quality evaluation
		prevHigh = high; 	// previous values for quality evaluation
		prevMid = mid; 		// previous values for quality evaluation

		// get mid-point for binary search 
		mid = calcMid(high, low);

		printf("\n%3d  %3d   %3d  ", low, mid, high);

		/* QA evaluation of algorithm, expected values:"
		   low <= mid <= high
		*/
		if (mid < low || mid > high) // ERROR: out of range 
		{
			printf("ERROR: mid is outside low...high range\n");
			errorsFound++; // increment error counter
			return false;  // target not found due to error
			// ********************** unconventional exit ****************************************
		}

		if (target < mid)		// target is LT mid point
		{
			high = mid - 1;			// new high is LT mid
		}
		else if (target > mid)  // target is GT mid point
		{
			low = mid + 1;			// new low is GT mid
		}
		else	// mid == target found at calculated mid-point 
		{
			printf("%d FOUND in %d tries\n", mid, counter);
			totalSearches += counter; // accumulate for average number of tries in successful searches
			countSuccessfulSearches++; // count number of successful searches 
			return true; // target found
			// ********************** unconventional exit ****************************************
		}
	} while (low <= high); // post-test loop. repeat search if a gap between low and high. not found cases will result in low/high values crossing over each other.

	printf("%d NOT found in %d tries\n", target, counter);
	countUnsuccessfulSearches++;
	return false; // valid case of target not found
}
